/*
 *    _           _             _   _
 *   (_) __ _  __| | __ _ _ __ | |_(_)_   _____
 *   | |/ _` |/ _` |/ _` | '_ \| __| \ \ / / _ \
 *   | | (_| | (_| | (_| | |_) | |_| |\ V /  __/
 *  _/ |\__,_|\__,_|\__,_| .__/ \__|_| \_/ \___|
 * |__/                  |_|
 *
 * This file is part of the Maverick Synergy Hotfixes Java SSH API
 *
 * Unauthorized copying of this file, via any medium is strictly prohibited.
 *
 * Copyright (C) 2002-2021 JADAPTIVE Limited - All Rights Reserved
 *
 * Use of this software may also be covered by third-party licenses depending on the choices you make about what features to use.
 *
 * Please visit the link below to see additional third-party licenses and copyrights
 *
 * https://www.jadaptive.com/app/manpage/en/article/1565029/What-third-party-dependencies-does-the-Maverick-Synergy-API-have
 */

package com.sshtools.common.files.nio;
import java.io.IOException;
import java.nio.file.attribute.BasicFileAttributeView;
import java.nio.file.attribute.FileAttributeView;
import java.nio.file.attribute.FileTime;
import java.util.LinkedHashMap;
import java.util.Map;

public class AbstractFileAttributeView implements BasicFileAttributeView {
	private static enum Attribute {
		attributes, certificates, contentEncoding, contentType, creationTime, fileKey, isDirectory, isOther, isRegularFile, isSymbolicLink, lastAccessTime, lastModifiedTime, size
	};

	@SuppressWarnings("unchecked")
	static <V extends FileAttributeView> V get(AbstractFilePath path, Class<V> type) {
		if (type == null)
			throw new NullPointerException();
		if (type == BasicFileAttributeView.class)
			try {
				return (V) new AbstractFileBasicAttributes(path.getAbstractFile());
			} catch (IOException e) {
			}
		return null;
	}
	
	static AbstractFileAttributeView get(AbstractFilePath path, String type) {
		if (type == null)
			throw new NullPointerException();
		if (type.equals("basic"))
			return new AbstractFileAttributeView(path);
		return null;
	}

	private final AbstractFilePath path;

	private AbstractFileAttributeView(AbstractFilePath path) {
		this.path = path;
	}

	@Override
	public String name() {
		return "basic";
	}

	@Override
	public AbstractFileBasicAttributes readAttributes() throws IOException {
		return path.getAttributes();
	}

	@Override
	public void setTimes(FileTime lastModifiedTime, FileTime lastAccessTime, FileTime createTime) throws IOException {
		// TODO
	}

	Object attribute(Attribute id, AbstractFileBasicAttributes attributes) {
		switch (id) {
		case size:
			return attributes.size();
		case creationTime:
			return attributes.creationTime();
		case lastAccessTime:
			return attributes.lastAccessTime();
		case lastModifiedTime:
			return attributes.lastModifiedTime();
		case isDirectory:
			return attributes.isDirectory();
		case isRegularFile:
			return attributes.isRegularFile();
		case isSymbolicLink:
			return attributes.isSymbolicLink();
		case isOther:
			return attributes.isOther();
		case fileKey:
			return attributes.fileKey();
		default:
		}
		return null;
	}

	Map<String, Object> readAttributes(String attributes) throws IOException {
		AbstractFileBasicAttributes zfas = readAttributes();
		LinkedHashMap<String, Object> map = new LinkedHashMap<>();
		if ("*".equals(attributes)) {
			for (Attribute id : Attribute.values()) {
				try {
					map.put(id.name(), attribute(id, zfas));
				} catch (IllegalArgumentException x) {
				}
			}
		} else {
			String[] as = attributes.split(",");
			for (String a : as) {
				try {
					map.put(a, attribute(Attribute.valueOf(a), zfas));
				} catch (IllegalArgumentException x) {
				}
			}
		}
		return map;
	}

	void setAttribute(String attribute, Object value) throws IOException {
		try {
			if (Attribute.valueOf(attribute) == Attribute.lastModifiedTime)
				setTimes((FileTime) value, null, null);
			return;
		} catch (IllegalArgumentException x) {
		}
		throw new UnsupportedOperationException("'" + attribute + "' is unknown or read-only attribute");
	}
}